// IcoFile.cpp: implementation of the CIcoFile class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BmpFileView.h"
#include "IcoFile.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CIcoFile::CIcoFile():
	m_pEntries(NULL),
	m_pImages(NULL)
{

}

CIcoFile::~CIcoFile()
{
	this->Clear();
}


void CIcoFile::Clear()
{
	//entries һͷż
	if(m_pEntries != NULL)
	{
		::free(m_pEntries);
		m_pEntries = NULL;
	}

	//images Ҫͷ
	if(m_pImages != NULL)
	{
		int i;
		for(i=0; i<this->m_imgCount; i++)
		{
			if(m_pImages[i].lpInfo != NULL)
				::free(m_pImages[i].lpInfo);

			if(m_pImages[i].lpXOR != NULL)
				::free(m_pImages[i].lpXOR);

			if(m_pImages[i].lpAND != NULL)
				::free(m_pImages[i].lpAND);
		}
		::free(m_pImages);
		m_pImages = NULL;
	}
	this->m_imgCount = 0;
}

//  ico ļΪ˴ļֱۣҼļϷ쳣
BOOL CIcoFile::LoadFile(LPCTSTR lpszFileName)
{
	int i;
	CFile file;
	if(file.Open(lpszFileName, CFile::modeRead))
	{
		//ԭеڴ
		this->Clear();
		file.Read(&this->m_fileHeader, sizeof(ICONFILEHEADER));
		this->m_imgCount = this->m_fileHeader.idCount; //ͼ꺬еͼ

		//ڴ-entries
		this->m_pEntries = (LPICONDIRENTRY)::malloc(sizeof(ICONDIRENTRY)*this->m_imgCount);
		//ڴ-images
		this->m_pImages = (LPICONIMAGE)::malloc(sizeof(ICONIMAGE)*this->m_imgCount);
		//-BITMAPINFO
		for(i =0 ; i < this->m_imgCount; i++)
		{
			this->m_pImages[i].lpInfo = (LPBITMAPINFO)::malloc(sizeof(BITMAPINFO));
		}

		//ȡ-entries
		file.Read(this->m_pEntries, sizeof(ICONDIRENTRY)*this->m_imgCount);

		//ѭȡ-images
		for(i = 0; i < this->m_imgCount; i++)
		{
			//====================================================
			//		1. bitmap_info_header
			//====================================================
			file.Read(this->m_pImages[i].lpInfo, sizeof(BITMAPINFOHEADER));
			//calc palette size
			this->m_pImages[i].nPaletteSize = this->CalcPaletteSize(
				this->m_pImages[i].lpInfo->bmiHeader.biBitCount,
				this->m_pImages[i].lpInfo->bmiHeader.biClrUsed,
				this->m_pImages[i].lpInfo->bmiHeader.biCompression
				);

			//====================================================
			//		2. palette (bmiColors)
			//====================================================
			if(this->m_pImages[i].nPaletteSize > 0)
			{
				//realloc bitmap_info
				this->m_pImages[i].lpInfo = (LPBITMAPINFO)::realloc(
					this->m_pImages[i].lpInfo,
					sizeof(BITMAPINFOHEADER) + sizeof(RGBQUAD)*(this->m_pImages[i].nPaletteSize)
					);

				file.Read(&this->m_pImages[i].lpInfo->bmiColors,
					sizeof(RGBQUAD)*(this->m_pImages[i].nPaletteSize)
					);
			}

			//====================================================
			//		3. XOR (ͼʾλͼ)
			//====================================================
			this->m_pImages[i].nLineSize_XOR = this->CalcLineSize(
				this->m_pImages[i].lpInfo->bmiHeader.biBitCount,
				this->m_pImages[i].lpInfo->bmiHeader.biWidth
				);

			//ע⣺entryͼʵ߶ȣǽͼСʱЧ
			//ǱȡbitmapInfoHeaderĸ߶ȣҪ2
			this->m_pImages[i].nSize_XOR = this->m_pImages[i].nLineSize_XOR 
				* (this->m_pImages[i].lpInfo->bmiHeader.biHeight/2);

			//ڴ
			this->m_pImages[i].lpXOR = ::malloc(this->m_pImages[i].nSize_XOR);
			file.Read(this->m_pImages[i].lpXOR, this->m_pImages[i].nSize_XOR);

			//====================================================
			//		4. AND (ڰ׵ɫλͼͼ͸)
			//====================================================
			this->m_pImages[i].nLineSize_AND = this->CalcLineSize(
				1, //bpp = 1
				this->m_pImages[i].lpInfo->bmiHeader.biWidth
				);

			//ע⣺entryͼʵ߶ȣԳͼΪ0
			//bitmapInfoHeader еĸ߶ȣֵΪ׼
			this->m_pImages[i].nSize_AND = this->m_pImages[i].nLineSize_AND 
				* (this->m_pImages[i].lpInfo->bmiHeader.biHeight/2);

			//ڴ
			this->m_pImages[i].lpAND = ::malloc(this->m_pImages[i].nSize_AND);
			file.Read(this->m_pImages[i].lpAND, this->m_pImages[i].nSize_AND);
		}
		file.Close();
		return TRUE;
	}
	return FALSE;
}

//ȡָͼԴС
size_t CIcoFile::GetSizeOfImage(int index)
{
	size_t nSize = 0;
	if(index < 0 || index >= this->m_imgCount || this->m_pImages == NULL)
		return nSize;
	
	nSize += sizeof(BITMAPINFOHEADER);
	nSize += sizeof(RGBQUAD) * m_pImages[index].nPaletteSize;
	nSize += m_pImages[index].nSize_AND;
	nSize += m_pImages[index].nSize_XOR;

	return nSize;
}

//ȡͼԴܴС
size_t CIcoFile::GetSizeOfAllImages()
{
	size_t i, nSize = 0;
	for(i=0; i < m_imgCount; i++)
		nSize += this->GetSizeOfImage(i);

	return nSize;
}

